@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
 
/* Horizontal enabled */
.autorepeatbutton-up[orient="horizontal"],
.scrollbutton-up[orient="horizontal"] {
  list-style-image: url("chrome://global/skin/arrow/arrow-lft-sharp.gif");
  -moz-image-region: auto; /* cut off inheritance */
}
 
.autorepeatbutton-down[orient="horizontal"],
.scrollbutton-down[orient="horizontal"] {
  list-style-image: url("chrome://global/skin/arrow/arrow-rit-sharp.gif");
  -moz-image-region: auto; /* cut off inheritance */
}

/* Horizontal disabled */
.autorepeatbutton-up[orient="horizontal"][disabled="true"],
.scrollbutton-up[orient="horizontal"][disabled="true"] {
  list-style-image: url("chrome://global/skin/arrow/arrow-lft-dis.gif");
  -moz-image-region: auto; /* cut off inheritance */
}
 
.autorepeatbutton-down[orient="horizontal"][disabled="true"],
.scrollbutton-down[orient="horizontal"][disabled="true"] {
  list-style-image: url("chrome://global/skin/arrow/arrow-rit-dis.gif");
  -moz-image-region: auto; /* cut off inheritance */
}

/* Vertical enabled */
.autorepeatbutton-up:not([orient="horizontal"]) {
  padding-bottom: 5px;
}

.autorepeatbutton-down:not([orient="horizontal"]) {
  padding-top: 5px;
}

.autorepeatbutton-up:not([orient="horizontal"]) > .autorepeatbutton-icon {
  -moz-transform: rotate(-90deg);
}

.autorepeatbutton-down:not([orient="horizontal"]) > .autorepeatbutton-icon {
  -moz-transform: rotate(90deg);
}

.scrollbutton-up {
  list-style-image: url("chrome://global/skin/arrow/arrow-up-sharp.gif");
  -moz-image-region: auto; /* cut off inheritance */
}

.scrollbutton-down {
  list-style-image: url("chrome://global/skin/arrow/arrow-dn-sharp.gif");
  -moz-image-region: auto; /* cut off inheritance */
}

/* Vertical disabled */
.autorepeatbutton-up[disabled="true"]:not([orient="horizontal"]),
.autorepeatbutton-down[disabled="true"]:not([orient="horizontal"]) {
  -moz-image-region: rect(0, 24px, 10px, 16px);
}

.scrollbutton-up[disabled="true"] {
  list-style-image: url("chrome://global/skin/arrow/arrow-up-dis.gif");
  -moz-image-region: auto; /* cut off inheritance */
}

.scrollbutton-down[disabled="true"] {
  list-style-image: url("chrome://global/skin/arrow/arrow-dn-dis.gif");
  -moz-image-region: auto; /* cut off inheritance */
}

.scrollbutton-up > .toolbarbutton-text,
.scrollbutton-down > .toolbarbutton-text {
  display: none;
}

autorepeatbutton {
  list-style-image: url("chrome://global/skin/menu/menu-arrow.png");
  -moz-image-region: rect(0, 8px, 10px, 0);
  -moz-box-align: center;
  -moz-box-pack: center;
  padding: 1px;
}
